#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of music related metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
#!/usr/bin/env python
# encoding: utf-8

from tests.__common__ import *


TESTCASES = [{
    # {{{
    'name': 'musicbrainz',
    'data': [{
        'options': {
            'get_type': 'albumlist',
            'artist': 'Equilibrium',
            'number': 4
            },
        'expect': lambda r: len(r) == 4
        }, {
        'options':  {
            'get_type': 'tracklist',
            'artist': 'Benele Banga Baracka'
            },
        'expect': len_equal_0
        }],
    }]
    # }}}



#!/usr/bin/env python
# encoding: utf-8

"""#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of musicrelated metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
"""

import sys

if __name__ == '__main__':
    for source in sys.argv[1:]:
        with open(source, 'r') as handle:
            content = __doc__ + handle.read()

        with open(source, 'w') as handle:
            handle.write(content)

    print(sys.argv)



#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of music related metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
import text_table

LEFT_TOP = (12, 'O/P')


class AsciiTable(object):
    'A cell based approach to drawing ascii tables'

    def __init__(self, pad_char = '?'):
        'pad_char is the char used for unset cells'
        self._rowdesc = set()
        self._columns = [LEFT_TOP]
        self._celltup = []
        self._padchar = pad_char

    def add(self, column, rowdesc, cell):
        'Add the text cell. The position is defined by column and rowdesc'
        column_tuple = (len(column), column)
        if column_tuple not in self._columns:
            self._columns.append((len(column), column))

        self._rowdesc.add(rowdesc)
        self._celltup.append((cell, rowdesc, column))

    def draw(self):
        'Draw previously added cells'
        columnlist = list(self._columns)
        table = text_table.TextTable(*columnlist)
        for rd in list(self._rowdesc):
            row = [rd] + [self._padchar] * (len(columnlist) - 1)
            for cell in self._celltup:
                # Check if the row description fits
                if cell[1] == rd:
                    cell_pos = 0
                    # Now find the corresponding column
                    for col in columnlist:
                        if cell[2] == col[1]:
                            row[cell_pos] = cell[0]
                        cell_pos += 1

            table.row(*row)

        return table.draw()


if __name__ == '__main__':
    at = AsciiTable(pad_char='*')
    at.add('Axxis', 'lyricswiki', '1')
    at.add('Adele', 'lyricswiki', '2')
    at.add('Pussycat Dolls',  'albumart', '3')
    at.add('Adele', 'albumart', '4')
    print(at.draw())



#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of music related metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
#!/usr/bin/env python
# encoding: utf-8

import plyr
import renderer
import termcolor
import threading
import os


PRINT_CHARS = [('Y', 'green'), ('X', 'red'), ('#', 'yellow')]
THREAD_LIST = []
DRAW_LOCK = threading.Lock()


def _run_query(provider, canvas, options, expect):
    'Do the actual plyr work'
    draw_string = 'X'

    # Only block realted to plyr.
    qry = plyr.Query(**options)
    qry.providers = [provider]
    results = qry.commit()

    # Write results to results/ folder
    ctr = 0
    for item in results:
        item.write(os.path.join('results', '{art}_{alb}_{tit}_{typ}_{prv}_{nbr}'.format(
            art = options.get('artist', ''),
            alb = options.get('album', ''),
            tit = options.get('title', ''),
            typ = options.get('get_type', ''),
            prv = provider,
            nbr = str(ctr)
            )))
        ctr += 1

    if expect(results):
        draw_string = 'Y'

    # Make sure no add() is done in parallel
    DRAW_LOCK.acquire()
    canvas.add(options['artist'], provider, draw_string)
    DRAW_LOCK.release()


def test_by_dictionary(testcases, includes=[], colored=True):
    'Input a dictionary with testcases and print the results (colored?)'
    canvas = renderer.AsciiTable(PRINT_CHARS[2][0])
    THREAD_LIST = []

    # Go through all testcases, and start a thread fro them
    for testcase in testcases:
        if len(includes) == 0 or testcase['name'] in includes:
            for subcase in testcase['data']:
                qry_thread = threading.Thread(target=_run_query,
                        args=(testcase['name'], canvas,
                            subcase['options'], subcase['expect']))

                THREAD_LIST.append(qry_thread)
                qry_thread.start()

    # Wait for all of them to join, and remove them eventually.
    for thread in THREAD_LIST:
        thread.join()

    # Make sure to be reentrant
    THREAD_LIST = []

    # Render the table
    block = canvas.draw()

    # Colorize it maybe
    if colored:
        for char_color in PRINT_CHARS:
            block = block.replace(char_color[0], termcolor.colored(
                char_color[0], color=char_color[1]))

    # Finally, print it.
    print(block)



#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of music related metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
#!/usr/bin/env python
# encoding: utf-8

from tests.__common__ import *

not_found_options = {
        'get_type': 'similarsongs',
        'artist': 'The band they called Horse',
        'title': 'The album you all know.'
        }

TESTCASES = [{
    # {{{
    'name': 'lastfm',
    'data': [{
        'options': {
            'get_type': 'similarsongs',
            'artist': 'Equilibrium',
            'title': 'Wurzelbert'
            },
        'expect': len_greater_0
        }, {
        'options': {
            'get_type': 'similarsongs',
            'artist': 'In Flames',
            'title': 'Trigger',
            'number': 3
            },
        'expect': lambda r: len(r) == 3
        }, {
        'options': not_found_options,
        'expect': len_equal_0
        }],
    },
    # }}}
]



#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of music related metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
#!/usr/bin/env python

'''text_table - Simple Eyecandy ASCII Tables

>>> from text_table import TextTable
>>> t = TextTable(5, 10)
>>> t.row('foobarbaz', 'yadayadayada')
>>> print t.draw()

.-------+------------.
| foob- | yadayaday- |
| arbaz | ada        |
'-------+------------'

>>> t = TextTable((5, 'abc'), (10, 'def'))
>>> t.row('foobarbaz', 'yadayadayada')
>>> print t.draw()

.-------+------------.
| abc   | def        |
+-------+------------+
| foob- | yadayaday- |
| arbaz | ada        |
'-------+------------'



Ported from Sebastian Riedel's Text::SimpleTable[1] to Python by Swaroop C H[2].
License is same as that of Text::SimpleTable - Artistic or GPL license.

[1]: http://search.cpan.org/~sri/Text-SimpleTable/lib/Text/SimpleTable.pm
[2]: http://www.swaroopch.info
'''

__version__ = '0.02'

TOP_LEFT        = '.-'
TOP_BORDER      = '-'
TOP_SEPARATOR   = '-+-'
TOP_RIGHT       = '-.'

MIDDLE_LEFT     = '+-'
MIDDLE_BORDER   = '-'
MIDDLE_SEPARATOR= '-+-'
MIDDLE_RIGHT    = '-+'

LEFT_BORDER     = '| '
SEPARATOR       = ' | '
RIGHT_BORDER    = ' |'

BOTTOM_LEFT     = "'-"
BOTTOM_SEPARATOR= '-+-'
BOTTOM_BORDER   = '-'
BOTTOM_RIGHT    = "-'"

WRAP            = '-'


class TextTable:
    '''Text ASCII table drawing.'''

    def __init__(self, *args, **kwargs):
        cache = []
        max = 0
        for arg in args:
            if isinstance(arg, tuple) or isinstance(arg, list):
                assert len(arg) == 2
                width, name = arg
            else:
                assert isinstance(arg, int)
                width = arg
                name = None

            if name is None or len(name) == 0:
                title = []
            else:
                title = self._wrap(name, width)

            col = [width, [], title]

            if max < len(col[2]):
                max = len(col[2])

            cache.append(col)

        for col in cache:
            while len(col[2]) < max:
                col[2].append('')

        self.columns = cache


    def row(self, *args, **kwargs):
        texts = list(args)
        size = len(self.columns) - 1
        if size < 0:
            return
        for i in range(0, size):
            if size <= len(texts):
                break
            texts.append('')

        cache = []
        max = 0
        for i in range(0, size+1):
            text = texts.pop(0)
            column = self.columns[i]
            width = column[0]
            pieces = self._wrap(text, width)
            cache.append(pieces)
            if max < len(pieces):
                max = len(pieces)

        for col in cache:
            while len(col) < max:
                col.append('')

        for i in range(0, size+1):
            column = self.columns[i]
            store = column[1]
            store.extend(cache[i])


    def draw(self):
        if self.columns is None or len(self.columns) == 0:
            return

        rows = len(self.columns[0][1]) - 1
        columns = len(self.columns) - 1
        output = ''

        # Top border
        for j in range(0, columns+1):
            column = self.columns[j]
            width = column[0]
            text = TOP_BORDER * width
            if j == 0 and columns == 0:
                text = TOP_LEFT + text + TOP_RIGHT
            elif j == 0:
                text = TOP_LEFT + text + TOP_SEPARATOR
            elif j == columns:
                text = text + TOP_RIGHT
            else:
                text = text + TOP_SEPARATOR
            output += text
        output += '\n'

        title = 0
        for column in self.columns:
            if title < len(column[2]):
                title = len(column[2])

        if title:
            for i in range(0, title):
                for j in range(0, columns+1):
                    column = self.columns[j]
                    width = column[0]
                    text = column[2][i] or ''
                    format = '%%-%ds' % width       # "%-${width}s"
                    text = format % text
                    if j == 0 and columns == 0:
                        text = LEFT_BORDER + text + RIGHT_BORDER
                    elif j == 0:
                        text = LEFT_BORDER + text + SEPARATOR
                    elif j == columns:
                        text = text + RIGHT_BORDER
                    else:
                        text = text + SEPARATOR
                    output += text
                output += '\n'

            # Title separator
            for j in range(0, columns+1):
                column = self.columns[j]
                width = column[0]
                text = MIDDLE_BORDER * width
                if j == 0 and columns == 0:
                    text = MIDDLE_LEFT + text + MIDDLE_RIGHT
                elif j == 0:
                    text = MIDDLE_LEFT + text + MIDDLE_SEPARATOR
                elif j == columns:
                    text = text + MIDDLE_RIGHT
                else:
                    text = text + MIDDLE_SEPARATOR
                output += text
            output += '\n'

        # Rows
        for i in range(0, rows+1):
            for j in range(0, columns+1):
                column = self.columns[j]
                width = column[0]
                text = column[1][i] or ''
                format = '%%-%ss' % width       # "%-${width}s"
                text = format % text
                if j == 0 and columns == 0:
                    text = LEFT_BORDER + text + RIGHT_BORDER
                elif j == 0:
                    text = LEFT_BORDER + text + SEPARATOR
                elif j == columns:
                    text = text + RIGHT_BORDER
                else:
                    text = text + SEPARATOR
                output += text
            output += '\n'

        # Bottom border
        for j in range(0, columns+1):
            column = self.columns[j]
            width = column[0]
            text = BOTTOM_BORDER * width
            if j == 0 and columns == 0:
                text = BOTTOM_LEFT + text + BOTTOM_RIGHT
            elif j == 0:
                text = BOTTOM_LEFT + text + BOTTOM_SEPARATOR
            elif j == columns:
                text = text + BOTTOM_RIGHT
            else:
                text = text + BOTTOM_SEPARATOR
            output += text
        output += '\n'

        return output


    def _wrap(self, text, width):
        cache = []
        parts = text.split('\n')
        for part in parts:
            while len(part) > width:
                subtext = part[:width-len(WRAP)]
                part = part[width-len(WRAP):]
                cache.append(subtext + WRAP)
            if part:
                cache.append(part)

        return cache


if __name__ == '__main__':
    # Example 1
    # t = TextTable(5, 10)
    # t.row('foobarbaz', 'yadayadayada')
    # print t.draw()

    # Example 2
    t = TextTable((5, 'abc'), (10, 'def'))
    t.row('foobarbaz', 'yadayadayada')
    print(t.draw())





#################################################################
# This file is part of glyr
# + a command-line tool and library to download various sort of music related metadata.
# + Copyright (C) [2011-2012]  [Christopher Pahl]
# + Hosted at: https://github.com/sahib/glyr
#
# glyr is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# glyr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with glyr. If not, see <http://www.gnu.org/licenses/>.
#################################################################
#!/usr/bin/env python
# encoding: utf-8

import os
import sys
import traceback
import executor

TESTS_DIR = 'tests'
USAGE = '''
Usage: runner.py [--help|-h] [--only|-o]

--help | -h   Print this text and exit.
--only | -o   Restrict test to certain providers and getters.

              Example:

                --only 'cover'
                --only 'cover|albumlist'
                --only 'cover:google'
                --only 'cover:google,lastfm'

With no arguments all tests are run.
'''


def parse_only_argument(argument):
    """
    Parse the argument of --only into a dictionary
    Example: --only 'albumlist:musicbrainz|cover:google,lastfm'
    """
    allowed_modules = {}

    for include_modules in sys.argv[2].split('|'):
        parseme = include_modules.split(':')
        module_name = parseme[0]
        if len(parseme) > 1:
            allowed_modules[module_name] = [x.strip() for x in parseme[1].split(',')]
        else:
            allowed_modules[module_name] = []

    return allowed_modules


def main():
    """
    Import all testfiles and run some of them.
    """

    if '--help' in sys.argv or '-h' in sys.argv:
        print(USAGE)
        sys.exit(0)

    test_modules = []
    allowed_modules = {}

    # Create a results dir
    try:
        os.mkdir('results')
    except OSError:
        pass

    # Try to import all files in tests/
    for test_file in os.listdir(TESTS_DIR):
        if not test_file.startswith('__'):
            full_path = '.'.join([TESTS_DIR, test_file[:-3]])
            try:
                test_modules.append((full_path, __import__(full_path, fromlist=['TESTCASES'])))
            except:
                print('Error while loading', full_path, traceback.format_exc())
                sys.exit(-1)

    # Parse --only string, and pass it further
    if len(sys.argv) > 2 and ('--only' in sys.argv or '-o' in sys.argv):
        allowed_modules = parse_only_argument(sys.argv[2])

    # Now run through them, and execute all specified ones.
    for module in test_modules:
        module_name = module[0].split('.')[-1]
        if len(allowed_modules) == 0 or module_name in allowed_modules:
            print('Running:', module[0], ':\n')
            try:
                executor.test_by_dictionary(module[1].TESTCASES,
                        includes=allowed_modules.get(module_name) or [],
                        colored=True)
            except:
                print('Error while executing tests:', full_path,
                        traceback.format_exc())
                sys.exit(-2)
            print('')

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        print('Interrupted.')
